use std::{fmt::Display, io::{BufWriter, Write}, path::PathBuf};
use anyhow::bail;
use bfox_parser::types::token::Token;
use clap::ValueEnum;

mod targets;

#[allow(non_camel_case_types)]
#[derive(Debug, Default, Clone, ValueEnum)]
pub enum AsmType {
    #[default]
    X86_64_NASM
}

impl Display for AsmType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{self:?}")
    }
}

pub struct AsmGen {
    asm_type: AsmType
}


impl AsmGen {
    pub fn new() -> Self {
        Self {
            asm_type: Default::default(),
        }
    }

    pub fn set_asm_type(&mut self, t: AsmType) {
        self.asm_type = t;
    }

    pub fn generate(&mut self, tokens: Vec<Token>, obj_path: PathBuf) -> anyhow::Result<()> {
        let asm_path = obj_path.with_extension("s");

        let fd_ = std::fs::File::options()
                    .read(true)
                    .write(true)
                    .truncate(true)
                    .create(true)
                    .open(&asm_path);

        let fd;
        match fd_ {
            Ok(fd_) => fd = fd_,
            Err(e) => {
                log::error!("Failed to open file {asm_path:?}: {e}");
                bail!("");
            },
        }

        let mut bw = BufWriter::new(fd);

        match self.asm_type {
            AsmType::X86_64_NASM => targets::compile_x86_64_nasm(self, tokens, &mut bw)?,
        }

        bw.flush()?;

        Ok(())
    }
}

